"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const meta_1 = require("../meta");
const fs = require('fs');
const path = require('path');
const deepmerge = require('deepmerge');
const recommended = require('./recommended');
const parseSeverityLut = [
    'disable',
    'warn',
    'error',
];
function parseSeverity(value) {
    if (typeof value === 'number') {
        return value;
    }
    else {
        return parseSeverityLut.indexOf(value.toLowerCase());
    }
}
class Config {
    constructor(options) {
        this.config = {
            extends: [],
            rules: {},
        };
        this.mergeInternal(options || {});
        this.metaTable = null;
        const self = this;
        this.config.extends.forEach(function (ref) {
            const base = Config.fromFile(ref);
            self.config = base.mergeInternal(self.config);
        });
    }
    static empty() {
        return new Config();
    }
    static fromObject(options) {
        return new Config(options);
    }
    static fromFile(filename) {
        if (filename === 'htmlvalidate:recommended') {
            return Config.fromObject(recommended);
        }
        const json = require(filename);
        for (const key of ['extends', 'elements']) {
            if (!json[key])
                continue;
            json[key] = json[key].map((ref) => {
                return Config.expandRelative(ref, path.dirname(filename));
            });
        }
        return new Config(json);
    }
    static defaultConfig() {
        return new Config({
            extends: [],
            rules: {},
        });
    }
    merge(rhs) {
        return new Config(this.mergeInternal(rhs.config));
    }
    getMetaTable() {
        if (this.metaTable) {
            return this.metaTable;
        }
        this.metaTable = new meta_1.MetaTable();
        const source = this.config.elements || ['html5'];
        const root = path.resolve(__dirname, '..', '..');
        for (const entry of source) {
            const filename = `${root}/elements/${entry}.json`;
            if (fs.existsSync(filename)) {
                this.metaTable.loadFromFile(filename);
                continue;
            }
            this.metaTable.loadFromObject(require(entry));
        }
        return this.metaTable;
    }
    static expandRelative(src, currentPath) {
        if (src[0] === '.') {
            return path.normalize(`${currentPath}/${src}`);
        }
        return src;
    }
    mergeInternal(config) {
        this.config = deepmerge(this.config, config);
        return this.config;
    }
    get() {
        return Object.assign({}, this.config);
    }
    getRules() {
        const rules = Object.assign({}, this.config.rules || {});
        for (const name in rules) {
            let options = rules[name];
            if (!Array.isArray(options)) {
                options = [options, {}];
            }
            else if (options.length === 1) {
                options.push({});
            }
            options[0] = parseSeverity(options[0]);
            rules[name] = options;
        }
        return rules;
    }
}
Config.SEVERITY_DISABLED = 0;
Config.SEVERITY_WARN = 1;
Config.SEVERITY_ERROR = 2;
exports.Config = Config;
