"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = require("./config");
const parser_1 = require("./parser");
const reporter_1 = require("./reporter");
const lexer_1 = require("./lexer");
const fs = require('fs');
class HtmlValidate {
    constructor(options) {
        const defaults = config_1.Config.defaultConfig();
        this.globalConfig = defaults.merge(config_1.Config.fromObject(options || {}));
    }
    validateString(str) {
        const source = { data: str, filename: 'inline' };
        const config = this.getConfigFor(source.filename);
        return this.process(source, config, 'lint');
    }
    validateFile(filename, mode) {
        const text = fs.readFileSync(filename, { encoding: 'utf8' });
        const source = { data: text, filename };
        const config = this.getConfigFor(source.filename);
        return this.process(source, config, mode);
    }
    process(source, config, mode) {
        switch (mode) {
            case 'lint':
            case undefined:
                return this.parse(source, config);
            case 'dump-events':
                return this.dumpEvents(source, config);
            case 'dump-tokens':
                return this.dumpTokens(source);
            case 'dump-tree':
                return this.dumpTree(source, config);
            default:
                throw new Error(`Unknown mode "${mode}"`);
        }
    }
    parse(src, config) {
        const report = new reporter_1.Reporter();
        const rules = config.getRules();
        const parser = new parser_1.Parser(config);
        for (const name in rules) {
            const data = rules[name];
            this.loadRule(name, data, parser, report);
        }
        try {
            parser.parseHtml(src);
        }
        catch (e) {
            if (e instanceof lexer_1.InvalidTokenError) {
                report.addManual(e.location.filename, {
                    ruleId: undefined,
                    severity: config_1.Config.SEVERITY_ERROR,
                    message: e.message,
                    line: e.location.line,
                    column: e.location.column,
                });
            }
            else {
                throw e;
            }
        }
        return report.save();
    }
    getParserFor(source) {
        const config = this.getConfigFor(source.filename);
        return new parser_1.Parser(config);
    }
    dumpEvents(source, config) {
        const parser = new parser_1.Parser(config);
        const filtered = ['parent', 'children'];
        parser.on('*', (event, data) => {
            const strdata = JSON.stringify(data, (key, value) => {
                return filtered.indexOf(key) >= 0 ? '[truncated]' : value;
            }, 2);
            process.stdout.write(`${event}: ${strdata}\n`);
        });
        parser.parseHtml(source);
        return {
            valid: true,
            results: [],
        };
    }
    dumpTokens(source) {
        const lexer = new lexer_1.Lexer();
        for (const token of lexer.tokenize(source)) {
            const data = token.data ? token.data[0] : null;
            process.stdout.write(`TOKEN: ${lexer_1.TokenType[token.type]}
  Data: ${JSON.stringify(data)}
  Location: ${token.location.filename}:${token.location.line}:${token.location.column}
`);
        }
        return {
            valid: true,
            results: [],
        };
    }
    dumpTree(source, config) {
        const parser = new parser_1.Parser(config);
        const dom = parser.parseHtml(source);
        function decoration(node) {
            let output = '';
            if (node.hasAttribute('id')) {
                output += `#${node.getAttribute('id')}`;
            }
            if (node.hasAttribute('class')) {
                output += `.${node.classList.join('.')}`;
            }
            return output;
        }
        function printNode(node, level, sibling) {
            if (level > 0) {
                const indent = '  '.repeat(level - 1);
                const l = node.children.length > 0 ? '┬' : '─';
                const b = sibling < (node.parent.children.length - 1) ? '├' : '└';
                process.stdout.write(`${indent}${b}─${l} ${node.tagName}${decoration(node)}\n`);
            }
            else {
                process.stdout.write(`(root)\n`);
            }
            node.children.forEach((child, index) => printNode(child, level + 1, index));
        }
        printNode(dom.root, 0, 0);
        return {
            valid: true,
            results: [],
        };
    }
    getConfigFor(filename) {
        const loader = new config_1.ConfigLoader();
        const config = loader.fromTarget(filename);
        return this.globalConfig.merge(config);
    }
    loadRule(name, data, parser, report) {
        const [severity, options] = data;
        if (severity >= config_1.Config.SEVERITY_WARN) {
            let rule;
            try {
                rule = require(`./rules/${name}`);
            }
            catch (e) {
                rule = {
                    name: name,
                    init: (parser) => {
                        parser.on('dom:load', (event, report) => {
                            report(null, `Definition for rule '${name}' was not found`);
                        });
                    },
                };
            }
            rule.init(this.createProxy(parser, rule, severity, report), options);
        }
    }
    createProxy(parser, rule, severity, report) {
        return {
            on: function (event, callback) {
                parser.on(event, function (event, data) {
                    callback.call(rule, data, reportFunc);
                    function reportFunc(node, message, location) {
                        const where = location || data.location || node.location;
                        report.add(node, rule, message, severity, where);
                    }
                });
            },
        };
    }
}
exports.default = HtmlValidate;
