"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dom_1 = require("../dom");
const lexer_1 = require("../lexer");
const event_1 = require("../event");
class Parser {
    constructor(config) {
        this.config = config;
        this.event = new event_1.EventHandler();
        this.dom = new dom_1.DOMTree();
        this.peeked = undefined;
        this.metaTable = config.getMetaTable();
    }
    on(event, listener) {
        this.event.on(event, listener);
    }
    parseHtml(source) {
        this.dom = new dom_1.DOMTree();
        if (typeof source === 'string') {
            source = { data: source, filename: 'inline' };
        }
        this.trigger('dom:load', {
            location: {},
        });
        const lexer = new lexer_1.Lexer();
        const tokenStream = lexer.tokenize(source);
        let it = this.next(tokenStream);
        while (!it.done) {
            const token = it.value;
            switch (token.type) {
                case lexer_1.TokenType.TAG_OPEN:
                    this.consumeTag(token, tokenStream);
                    break;
                case lexer_1.TokenType.WHITESPACE:
                    this.trigger('whitespace', {
                        text: token.data[0],
                        location: token.location,
                    });
                    break;
                case lexer_1.TokenType.EOF:
                    this.closeTree(token);
                    break;
            }
            it = this.next(tokenStream);
        }
        this.dom.resolveMeta(this.metaTable);
        this.trigger('dom:ready', {
            document: this.dom,
            location: false,
        });
        return this.dom;
    }
    closeOptional(token) {
        const active = this.dom.getActive();
        if (!(active.meta && active.meta.implicitClosed)) {
            return false;
        }
        const tagName = token.data[2];
        const open = !token.data[1];
        const meta = (active.meta.implicitClosed || []);
        if (open) {
            return meta.indexOf(tagName) >= 0;
        }
        else {
            if (active.is(tagName)) {
                return false;
            }
            return active.parent.is(tagName) && meta.indexOf(active.tagName) >= 0;
        }
    }
    consumeTag(startToken, tokenStream) {
        const tokens = Array.from(this.consumeUntil(tokenStream, lexer_1.TokenType.TAG_CLOSE));
        const endToken = tokens.slice(-1)[0];
        const closeOptional = this.closeOptional(startToken);
        const parent = closeOptional ? this.dom.getActive().parent : this.dom.getActive();
        const node = dom_1.DOMNode.fromTokens(startToken, endToken, parent, this.metaTable);
        const open = !startToken.data[1];
        const close = !open || node.closed !== dom_1.NodeClosed.Open;
        if (closeOptional) {
            const active = this.dom.getActive();
            active.closed = dom_1.NodeClosed.ImplicitClosed;
            this.trigger('tag:close', {
                target: node,
                previous: active,
                location: startToken.location,
            });
            this.dom.popActive();
        }
        if (open) {
            this.dom.pushActive(node);
            this.trigger('tag:open', {
                target: node,
                location: startToken.location,
            });
        }
        for (let i = 0; i < tokens.length; i++) {
            const token = tokens[i];
            switch (token.type) {
                case lexer_1.TokenType.WHITESPACE:
                    break;
                case lexer_1.TokenType.ATTR_NAME:
                    this.consumeAttribute(node, token, tokens[i + 1]);
                    break;
            }
        }
        if (close) {
            const active = this.dom.getActive();
            if (!open) {
                node.closed = dom_1.NodeClosed.EndTag;
            }
            this.trigger('tag:close', {
                target: node,
                previous: active,
                location: endToken.location,
            });
            const voidClosed = !open && node.voidElement;
            if (!voidClosed) {
                this.dom.popActive();
            }
        }
    }
    consumeAttribute(node, token, next) {
        const key = token.data[1];
        let value = undefined;
        let quote = undefined;
        if (next && next.type === lexer_1.TokenType.ATTR_VALUE) {
            value = next.data[1];
            quote = next.data[2];
        }
        this.trigger('attr', {
            target: node,
            key,
            value,
            quote,
            location: token.location,
        });
        node.setAttribute(key, value);
    }
    *consumeUntil(tokenStream, search) {
        let it = this.next(tokenStream);
        while (!it.done) {
            const token = it.value;
            yield token;
            if (token.type === search)
                return;
            it = this.next(tokenStream);
        }
        throw Error('stream ended before consumeUntil finished');
    }
    next(tokenStream) {
        if (this.peeked) {
            const peeked = this.peeked;
            this.peeked = undefined;
            return peeked;
        }
        else {
            return tokenStream.next();
        }
    }
    peek(tokenStream) {
        if (this.peeked) {
            return this.peeked;
        }
        else {
            return this.peeked = tokenStream.next();
        }
    }
    trigger(event, data) {
        if (typeof data.location === 'undefined') {
            throw Error('Triggered event must contain location');
        }
        this.event.trigger(event, data);
    }
    closeTree(token) {
        let active;
        while ((active = this.dom.getActive()) && active.tagName) {
            this.trigger('tag:close', {
                target: undefined,
                previous: active,
                location: token.location,
            });
            this.dom.popActive();
        }
    }
}
exports.Parser = Parser;
